# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..types import bulk_enrich_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.shared.company_detailed import CompanyDetailed

__all__ = ["BulkResource", "AsyncBulkResource"]


class BulkResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BulkResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/TJC-LP/grata-python#accessing-raw-response-data-eg-headers
        """
        return BulkResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BulkResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/TJC-LP/grata-python#with_streaming_response
        """
        return BulkResourceWithStreamingResponse(self)

    def enrich(
        self,
        *,
        authorization: str,
        company_uids: List[str] | NotGiven = NOT_GIVEN,
        domains: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyDetailed:
        """
        Provide a set of up to 100 company domains or Grata-specific company IDs to
        return relevant firmographic data on requested companies.

        Args:
          company_uids: An array of unique alphanumeric Grata IDs for the companies.

          domains: An array of domains for the companies being enriched.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return self._post(
            "/api/v1.4/bulk/enrich/",
            body=maybe_transform(
                {
                    "company_uids": company_uids,
                    "domains": domains,
                },
                bulk_enrich_params.BulkEnrichParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyDetailed,
        )


class AsyncBulkResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBulkResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/TJC-LP/grata-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBulkResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBulkResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/TJC-LP/grata-python#with_streaming_response
        """
        return AsyncBulkResourceWithStreamingResponse(self)

    async def enrich(
        self,
        *,
        authorization: str,
        company_uids: List[str] | NotGiven = NOT_GIVEN,
        domains: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyDetailed:
        """
        Provide a set of up to 100 company domains or Grata-specific company IDs to
        return relevant firmographic data on requested companies.

        Args:
          company_uids: An array of unique alphanumeric Grata IDs for the companies.

          domains: An array of domains for the companies being enriched.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return await self._post(
            "/api/v1.4/bulk/enrich/",
            body=await async_maybe_transform(
                {
                    "company_uids": company_uids,
                    "domains": domains,
                },
                bulk_enrich_params.BulkEnrichParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyDetailed,
        )


class BulkResourceWithRawResponse:
    def __init__(self, bulk: BulkResource) -> None:
        self._bulk = bulk

        self.enrich = to_raw_response_wrapper(
            bulk.enrich,
        )


class AsyncBulkResourceWithRawResponse:
    def __init__(self, bulk: AsyncBulkResource) -> None:
        self._bulk = bulk

        self.enrich = async_to_raw_response_wrapper(
            bulk.enrich,
        )


class BulkResourceWithStreamingResponse:
    def __init__(self, bulk: BulkResource) -> None:
        self._bulk = bulk

        self.enrich = to_streamed_response_wrapper(
            bulk.enrich,
        )


class AsyncBulkResourceWithStreamingResponse:
    def __init__(self, bulk: AsyncBulkResource) -> None:
        self._bulk = bulk

        self.enrich = async_to_streamed_response_wrapper(
            bulk.enrich,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["CompanyDetailed", "Result"]


class Result(BaseModel):
    company_count: Optional[float] = None
    """Number of companies in the list"""

    created_date: Optional[str] = None
    """Date the list was created"""

    list_uid: Optional[str] = None
    """UID of list"""

    name: Optional[str] = None
    """List name"""

    updated_date: Optional[str] = None
    """Date the list was last updated"""


class CompanyDetailed(BaseModel):
    count: Optional[float] = None
    """Number of lists in the results"""

    page: Optional[float] = None
    """Current page of results"""

    pages: Optional[float] = None
    """Number of pages of results"""

    results: Optional[List[Result]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "SearchSimilarParams",
    "Headquarters",
    "HeadquartersExclude",
    "HeadquartersInclude",
    "IndustryClassifications",
    "Lists",
    "TermsInclude",
    "TermsIncludeGroup",
]


class SearchSimilarParams(TypedDict, total=False):
    authorization: Required[Annotated[str, PropertyInfo(alias="Authorization")]]

    business_models: List[
        Literal[
            "software",
            "software_enabled",
            "services",
            "hardware",
            "content_and_publishing",
            "investment_banks_and_business_brokers",
            "education",
            "directory",
            "job_site",
            "staffing_and_recruiting",
            "private_equity_and_venture_capital",
            "private_schools",
            "retailer",
            "manufacturer",
            "distributor",
            "producer",
            "marketplace",
            "hospitals_and_medical_centers",
            "colleges_and_universities",
            "government",
            "us_federal_agencies",
            "nonprofit_and_associations",
            "religious_institutions",
        ]
    ]
    """Business models to search on."""

    company_uid: str
    """Alphanumeric Grata ID for the company (case-sensitive)."""

    domain: str
    """Domain of the company for similar search.

    Protocol and path can be included. If both the domain and company_uid are
    specified, domain will be referenced.
    """

    employees_change: Iterable[float]
    """Range of % employee growth."""

    employees_change_time: Literal["month", "quarter", "six_month", "annual"]
    """The interval for employee growth rate."""

    employees_on_professional_networks_range: Iterable[float]
    """The range of employee counts listed on professional networks.

    Inputting 100,001 as the maximum value will search for all employee sizes above
    the minimum. [100,100001] will search for all companies with 100 or more
    employees
    """

    end_customer: List[
        Literal[
            "b2b",
            "b2c",
            "information_technology",
            "professional_services",
            "electronics",
            "commercial_and_residential_services",
            "hospitality_and_leisure",
            "media",
            "finance",
            "industrials",
            "transportation",
            "education",
            "agriculture",
            "healthcare",
            "government",
            "consumer_product_and_retail",
        ]
    ]
    """End vertical that the company sells to."""

    funding_size: Iterable[float]
    """Range of funding the company has received in USD.

    Ranges can only start and begin with the following values: 0, 5000000, 10000000,
    20000000, 50000000, 100000000, 200000000, 500000000, 500000001. 500000001
    equates to maximum.
    """

    funding_stage: List[
        Literal[
            "early_stage_funding", "late_stage_funding", "private_equity_backed", "other_funding", "pre_ipo_funding"
        ]
    ]

    grata_employees_estimates_range: Iterable[float]
    """The range of employee counts based on Grata Employee estimates.

    Inputting 100,001 as the maximum value will search for all employee sizes above
    the minimum. [100,100001] will search for all companies with 100 or more
    employees
    """

    headquarters: Headquarters
    """Headquarter locations supports all countries and US city/states.

    State cannot be left blank if city is populated. Country cannot be other than
    United States if searching for city/state.
    """

    industry_classifications: IndustryClassifications
    """Industry classification code for the company.

    Pass the industry NAICS code or Grata's specific software industry code listed
    in the mapping doc -
    https://grata.stoplight.io/docs/grata/branches/v1.3/42ptq2xej8i5j-software-industry-code-mapping
    """

    is_funded: bool
    """Indicates whether or not the company has received outside funding."""

    lists: Lists
    """Grata list IDs to search within.

    Default logic for include is "or", default logic for exclude is "and."
    """

    ownership: List[
        Literal[
            "bootstrapped",
            "investor_backed",
            "public",
            "public_subsidiary",
            "private_subsidiary",
            "private_equity",
            "private_equity_add_on",
        ]
    ]
    """Ownership types to search and sort on."""

    page_token: str

    terms_exclude: List[str]
    """Keywords to exclude from the search."""

    terms_include: TermsInclude
    """String used for keyword search. This is an array of keywords"""

    year_founded: Iterable[float]
    """Range of founding years."""


class HeadquartersExclude(TypedDict, total=False):
    city: Optional[str]

    country: str

    state: Optional[str]


class HeadquartersInclude(TypedDict, total=False):
    city: Optional[str]

    country: str

    state: Optional[str]


class Headquarters(TypedDict, total=False):
    exclude: Iterable[HeadquartersExclude]

    include: Iterable[HeadquartersInclude]


class IndustryClassifications(TypedDict, total=False):
    exclude: Iterable[float]

    include: Iterable[float]


class Lists(TypedDict, total=False):
    exclude: List[str]

    include: List[str]


class TermsIncludeGroup(TypedDict, total=False):
    terms: List[str]

    terms_depth: Literal["core", "mention"]

    terms_operator: Literal["any", "all"]


class TermsInclude(TypedDict, total=False):
    group_operator: Literal["any", "all"]

    groups: Iterable[TermsIncludeGroup]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import list_list_params, list_create_params, list_update_params, list_companies_params
from .._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.shared.company_detailed import CompanyDetailed

__all__ = ["ListsResource", "AsyncListsResource"]


class ListsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ListsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/TJC-LP/grata-python#accessing-raw-response-data-eg-headers
        """
        return ListsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ListsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/TJC-LP/grata-python#with_streaming_response
        """
        return ListsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        authorization: str,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyDetailed:
        """The create call will allow users to create a list in Grata.

        The list visibility
        will be set to organization and it will be visible in the Grata UI for all users
        within the account to view.

        Args:
          name: Name of list being created

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return self._post(
            "/api/v1.4/lists/",
            body=maybe_transform({"name": name}, list_create_params.ListCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyDetailed,
        )

    def retrieve(
        self,
        list_uid: str,
        *,
        authorization: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyDetailed:
        """
        Grata's List Details API returns details about the lists in your organization.
        Private lists will not be returned with this call.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not list_uid:
            raise ValueError(f"Expected a non-empty value for `list_uid` but received {list_uid!r}")
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return self._get(
            f"/api/v1.4/lists/{list_uid}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyDetailed,
        )

    def update(
        self,
        list_uid: str,
        *,
        authorization: str,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyDetailed:
        """Grata's List Name API updates the name of a list.

        Private lists are not eligible
        to be updated with this call.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not list_uid:
            raise ValueError(f"Expected a non-empty value for `list_uid` but received {list_uid!r}")
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return self._patch(
            f"/api/v1.4/lists/{list_uid}/",
            body=maybe_transform({"name": name}, list_update_params.ListUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyDetailed,
        )

    def list(
        self,
        *,
        authorization: str,
        name: str | NotGiven = NOT_GIVEN,
        page: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyDetailed:
        """
        Grata's Search Lists API enables you to search for and return details about the
        lists in your organization. Private lists will not be returned with this call.

        Args:
          name: List name

          page: The page of results to be returned

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return self._get(
            "/api/v1.4/lists/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "name": name,
                        "page": page,
                    },
                    list_list_params.ListListParams,
                ),
            ),
            cast_to=CompanyDetailed,
        )

    def delete(
        self,
        list_uid: str,
        *,
        authorization: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Delete List

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not list_uid:
            raise ValueError(f"Expected a non-empty value for `list_uid` but received {list_uid!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        extra_headers.update({"Authorization": authorization})
        return self._delete(
            f"/api/v1.4/lists/{list_uid}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def companies(
        self,
        list_uid: str,
        *,
        authorization: str,
        action: Literal["add", "remove"] | NotGiven = NOT_GIVEN,
        domains: List[str] | NotGiven = NOT_GIVEN,
        uids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyDetailed:
        """
        The modify call will allow users to add or remove companies from an exisiting
        list. Private lists are not eligible to be updated with this call.

        Args:
          domains: Domains to add or remove from a list (max of 500 permitted per call).

          uids: Grata company UIDs to add or remove from a list (max of 500 permitted per call).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not list_uid:
            raise ValueError(f"Expected a non-empty value for `list_uid` but received {list_uid!r}")
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return self._post(
            f"/api/v1.4/lists/{list_uid}/companies/",
            body=maybe_transform(
                {
                    "action": action,
                    "domains": domains,
                    "uids": uids,
                },
                list_companies_params.ListCompaniesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyDetailed,
        )


class AsyncListsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncListsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/TJC-LP/grata-python#accessing-raw-response-data-eg-headers
        """
        return AsyncListsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncListsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/TJC-LP/grata-python#with_streaming_response
        """
        return AsyncListsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        authorization: str,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyDetailed:
        """The create call will allow users to create a list in Grata.

        The list visibility
        will be set to organization and it will be visible in the Grata UI for all users
        within the account to view.

        Args:
          name: Name of list being created

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return await self._post(
            "/api/v1.4/lists/",
            body=await async_maybe_transform({"name": name}, list_create_params.ListCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyDetailed,
        )

    async def retrieve(
        self,
        list_uid: str,
        *,
        authorization: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyDetailed:
        """
        Grata's List Details API returns details about the lists in your organization.
        Private lists will not be returned with this call.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not list_uid:
            raise ValueError(f"Expected a non-empty value for `list_uid` but received {list_uid!r}")
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return await self._get(
            f"/api/v1.4/lists/{list_uid}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyDetailed,
        )

    async def update(
        self,
        list_uid: str,
        *,
        authorization: str,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyDetailed:
        """Grata's List Name API updates the name of a list.

        Private lists are not eligible
        to be updated with this call.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not list_uid:
            raise ValueError(f"Expected a non-empty value for `list_uid` but received {list_uid!r}")
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return await self._patch(
            f"/api/v1.4/lists/{list_uid}/",
            body=await async_maybe_transform({"name": name}, list_update_params.ListUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyDetailed,
        )

    async def list(
        self,
        *,
        authorization: str,
        name: str | NotGiven = NOT_GIVEN,
        page: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyDetailed:
        """
        Grata's Search Lists API enables you to search for and return details about the
        lists in your organization. Private lists will not be returned with this call.

        Args:
          name: List name

          page: The page of results to be returned

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return await self._get(
            "/api/v1.4/lists/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "name": name,
                        "page": page,
                    },
                    list_list_params.ListListParams,
                ),
            ),
            cast_to=CompanyDetailed,
        )

    async def delete(
        self,
        list_uid: str,
        *,
        authorization: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Delete List

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not list_uid:
            raise ValueError(f"Expected a non-empty value for `list_uid` but received {list_uid!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        extra_headers.update({"Authorization": authorization})
        return await self._delete(
            f"/api/v1.4/lists/{list_uid}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def companies(
        self,
        list_uid: str,
        *,
        authorization: str,
        action: Literal["add", "remove"] | NotGiven = NOT_GIVEN,
        domains: List[str] | NotGiven = NOT_GIVEN,
        uids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyDetailed:
        """
        The modify call will allow users to add or remove companies from an exisiting
        list. Private lists are not eligible to be updated with this call.

        Args:
          domains: Domains to add or remove from a list (max of 500 permitted per call).

          uids: Grata company UIDs to add or remove from a list (max of 500 permitted per call).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not list_uid:
            raise ValueError(f"Expected a non-empty value for `list_uid` but received {list_uid!r}")
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return await self._post(
            f"/api/v1.4/lists/{list_uid}/companies/",
            body=await async_maybe_transform(
                {
                    "action": action,
                    "domains": domains,
                    "uids": uids,
                },
                list_companies_params.ListCompaniesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyDetailed,
        )


class ListsResourceWithRawResponse:
    def __init__(self, lists: ListsResource) -> None:
        self._lists = lists

        self.create = to_raw_response_wrapper(
            lists.create,
        )
        self.retrieve = to_raw_response_wrapper(
            lists.retrieve,
        )
        self.update = to_raw_response_wrapper(
            lists.update,
        )
        self.list = to_raw_response_wrapper(
            lists.list,
        )
        self.delete = to_raw_response_wrapper(
            lists.delete,
        )
        self.companies = to_raw_response_wrapper(
            lists.companies,
        )


class AsyncListsResourceWithRawResponse:
    def __init__(self, lists: AsyncListsResource) -> None:
        self._lists = lists

        self.create = async_to_raw_response_wrapper(
            lists.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            lists.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            lists.update,
        )
        self.list = async_to_raw_response_wrapper(
            lists.list,
        )
        self.delete = async_to_raw_response_wrapper(
            lists.delete,
        )
        self.companies = async_to_raw_response_wrapper(
            lists.companies,
        )


class ListsResourceWithStreamingResponse:
    def __init__(self, lists: ListsResource) -> None:
        self._lists = lists

        self.create = to_streamed_response_wrapper(
            lists.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            lists.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            lists.update,
        )
        self.list = to_streamed_response_wrapper(
            lists.list,
        )
        self.delete = to_streamed_response_wrapper(
            lists.delete,
        )
        self.companies = to_streamed_response_wrapper(
            lists.companies,
        )


class AsyncListsResourceWithStreamingResponse:
    def __init__(self, lists: AsyncListsResource) -> None:
        self._lists = lists

        self.create = async_to_streamed_response_wrapper(
            lists.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            lists.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            lists.update,
        )
        self.list = async_to_streamed_response_wrapper(
            lists.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            lists.delete,
        )
        self.companies = async_to_streamed_response_wrapper(
            lists.companies,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BulkEnrichParams"]


class BulkEnrichParams(TypedDict, total=False):
    authorization: Required[Annotated[str, PropertyInfo(alias="Authorization")]]

    company_uids: List[str]
    """An array of unique alphanumeric Grata IDs for the companies."""

    domains: List[str]
    """An array of domains for the companies being enriched."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ListCreateParams"]


class ListCreateParams(TypedDict, total=False):
    authorization: Required[Annotated[str, PropertyInfo(alias="Authorization")]]

    name: str
    """Name of list being created"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["EnrichCreateParams"]


class EnrichCreateParams(TypedDict, total=False):
    authorization: Required[Annotated[str, PropertyInfo(alias="Authorization")]]

    company_uid: str
    """Unique alphanumeric Grata ID for the company (case-sensitive)."""

    domain: str
    """Domain of the company being enriched.

    Protocol and path can be included. If both the domain and company_uid are
    included, the domain will be referenced.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ListCompaniesParams"]


class ListCompaniesParams(TypedDict, total=False):
    authorization: Required[Annotated[str, PropertyInfo(alias="Authorization")]]

    action: Literal["add", "remove"]

    domains: List[str]
    """Domains to add or remove from a list (max of 500 permitted per call)."""

    uids: List[str]
    """
    Grata company UIDs to add or remove from a list (max of 500 permitted per call).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .shared import CompanyDetailed as CompanyDetailed
from .company_basic import CompanyBasic as CompanyBasic
from .list_list_params import ListListParams as ListListParams
from .bulk_enrich_params import BulkEnrichParams as BulkEnrichParams
from .list_create_params import ListCreateParams as ListCreateParams
from .list_update_params import ListUpdateParams as ListUpdateParams
from .enrich_create_params import EnrichCreateParams as EnrichCreateParams
from .search_create_params import SearchCreateParams as SearchCreateParams
from .list_companies_params import ListCompaniesParams as ListCompaniesParams
from .search_similar_params import SearchSimilarParams as SearchSimilarParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["CompanyBasic", "Company", "Result"]


class Company(BaseModel):
    company_uid: Optional[str] = None
    """Unique alphanumeric Grata ID for the company (case-sensitive)."""

    description: Optional[str] = None
    """Description of the company."""

    domain: Optional[str] = None
    """Domain of the company."""

    name: Optional[str] = None
    """Name of the company."""

    url: Optional[str] = None
    """URL to the company's Grata profile."""


class Result(BaseModel):
    company_uid: Optional[str] = None
    """Unique alphanumeric Grata ID for the company (case-sensitive)."""

    description: Optional[str] = None
    """Description of the company."""

    domain: Optional[str] = None
    """Domain of the company."""

    name: Optional[str] = None
    """Name of the company."""

    url: Optional[str] = None
    """URL to the company's Grata profile."""


class CompanyBasic(BaseModel):
    company: Optional[Company] = None

    count: Optional[float] = None
    """Total number of companies in the search."""

    page_token: Optional[str] = None
    """Page token used for pagination."""

    results: Optional[List[Result]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from grata import Grata, AsyncGrata
from tests.utils import assert_matches_type
from grata.types.shared import CompanyDetailed

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBulk:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_enrich(self, client: Grata) -> None:
        bulk = client.bulk.enrich(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert_matches_type(CompanyDetailed, bulk, path=["response"])

    @parametrize
    def test_method_enrich_with_all_params(self, client: Grata) -> None:
        bulk = client.bulk.enrich(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            company_uids=["GAGRYBUR", "UFFY5AZY"],
            domains=["okta.com", "slack.com"],
        )
        assert_matches_type(CompanyDetailed, bulk, path=["response"])

    @parametrize
    def test_raw_response_enrich(self, client: Grata) -> None:
        response = client.bulk.with_raw_response.enrich(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bulk = response.parse()
        assert_matches_type(CompanyDetailed, bulk, path=["response"])

    @parametrize
    def test_streaming_response_enrich(self, client: Grata) -> None:
        with client.bulk.with_streaming_response.enrich(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bulk = response.parse()
            assert_matches_type(CompanyDetailed, bulk, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncBulk:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_enrich(self, async_client: AsyncGrata) -> None:
        bulk = await async_client.bulk.enrich(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert_matches_type(CompanyDetailed, bulk, path=["response"])

    @parametrize
    async def test_method_enrich_with_all_params(self, async_client: AsyncGrata) -> None:
        bulk = await async_client.bulk.enrich(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            company_uids=["GAGRYBUR", "UFFY5AZY"],
            domains=["okta.com", "slack.com"],
        )
        assert_matches_type(CompanyDetailed, bulk, path=["response"])

    @parametrize
    async def test_raw_response_enrich(self, async_client: AsyncGrata) -> None:
        response = await async_client.bulk.with_raw_response.enrich(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bulk = await response.parse()
        assert_matches_type(CompanyDetailed, bulk, path=["response"])

    @parametrize
    async def test_streaming_response_enrich(self, async_client: AsyncGrata) -> None:
        async with async_client.bulk.with_streaming_response.enrich(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bulk = await response.parse()
            assert_matches_type(CompanyDetailed, bulk, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from grata import Grata, AsyncGrata
from grata.types import CompanyBasic
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSearch:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Grata) -> None:
        search = client.search.create(
            authorization="Authorization",
        )
        assert_matches_type(CompanyBasic, search, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Grata) -> None:
        search = client.search.create(
            authorization="Authorization",
            business_models=["software"],
            company_uid="26AJRCR2",
            domain="slack.com",
            employees_change=[-50],
            employees_change_time="month",
            employees_on_professional_networks_range=[0],
            end_customer=["b2b", "b2c"],
            funding_size=[0, 100000000],
            funding_stage=["early_stage_funding"],
            grata_employees_estimates_range=[10, 100],
            headquarters={
                "exclude": [
                    {
                        "city": "New York",
                        "country": "United States",
                        "state": "New York",
                    }
                ],
                "include": [
                    {
                        "city": "Los Angeles ",
                        "country": "United States",
                        "state": "California",
                    }
                ],
            },
            industry_classifications={
                "exclude": [0],
                "include": [0],
            },
            is_funded=True,
            lists={
                "exclude": ["string"],
                "include": ["string"],
            },
            ownership=["bootstrapped"],
            page_token="",
            terms_exclude=["string"],
            terms_include={
                "group_operator": "any",
                "groups": [
                    {
                        "terms": ["messaging"],
                        "terms_depth": "core",
                        "terms_operator": "any",
                    }
                ],
            },
            year_founded=[1970, 2018],
        )
        assert_matches_type(CompanyBasic, search, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Grata) -> None:
        response = client.search.with_raw_response.create(
            authorization="Authorization",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = response.parse()
        assert_matches_type(CompanyBasic, search, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Grata) -> None:
        with client.search.with_streaming_response.create(
            authorization="Authorization",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = response.parse()
            assert_matches_type(CompanyBasic, search, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_similar(self, client: Grata) -> None:
        search = client.search.similar(
            authorization="Authorization",
        )
        assert_matches_type(CompanyBasic, search, path=["response"])

    @parametrize
    def test_method_similar_with_all_params(self, client: Grata) -> None:
        search = client.search.similar(
            authorization="Authorization",
            business_models=["software"],
            company_uid="26AJRCR2",
            domain="slack.com",
            employees_change=[-50],
            employees_change_time="month",
            employees_on_professional_networks_range=[0],
            end_customer=["b2b", "b2c"],
            funding_size=[0, 100000000],
            funding_stage=["early_stage_funding"],
            grata_employees_estimates_range=[10, 100],
            headquarters={
                "exclude": [
                    {
                        "city": "New York",
                        "country": "United States",
                        "state": "New York",
                    }
                ],
                "include": [
                    {
                        "city": "Los Angeles ",
                        "country": "United States",
                        "state": "California",
                    }
                ],
            },
            industry_classifications={
                "exclude": [0],
                "include": [0],
            },
            is_funded=True,
            lists={
                "exclude": ["string"],
                "include": ["string"],
            },
            ownership=["bootstrapped"],
            page_token="",
            terms_exclude=["string"],
            terms_include={
                "group_operator": "any",
                "groups": [
                    {
                        "terms": ["messaging"],
                        "terms_depth": "core",
                        "terms_operator": "any",
                    }
                ],
            },
            year_founded=[1970, 2018],
        )
        assert_matches_type(CompanyBasic, search, path=["response"])

    @parametrize
    def test_raw_response_similar(self, client: Grata) -> None:
        response = client.search.with_raw_response.similar(
            authorization="Authorization",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = response.parse()
        assert_matches_type(CompanyBasic, search, path=["response"])

    @parametrize
    def test_streaming_response_similar(self, client: Grata) -> None:
        with client.search.with_streaming_response.similar(
            authorization="Authorization",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = response.parse()
            assert_matches_type(CompanyBasic, search, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSearch:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncGrata) -> None:
        search = await async_client.search.create(
            authorization="Authorization",
        )
        assert_matches_type(CompanyBasic, search, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncGrata) -> None:
        search = await async_client.search.create(
            authorization="Authorization",
            business_models=["software"],
            company_uid="26AJRCR2",
            domain="slack.com",
            employees_change=[-50],
            employees_change_time="month",
            employees_on_professional_networks_range=[0],
            end_customer=["b2b", "b2c"],
            funding_size=[0, 100000000],
            funding_stage=["early_stage_funding"],
            grata_employees_estimates_range=[10, 100],
            headquarters={
                "exclude": [
                    {
                        "city": "New York",
                        "country": "United States",
                        "state": "New York",
                    }
                ],
                "include": [
                    {
                        "city": "Los Angeles ",
                        "country": "United States",
                        "state": "California",
                    }
                ],
            },
            industry_classifications={
                "exclude": [0],
                "include": [0],
            },
            is_funded=True,
            lists={
                "exclude": ["string"],
                "include": ["string"],
            },
            ownership=["bootstrapped"],
            page_token="",
            terms_exclude=["string"],
            terms_include={
                "group_operator": "any",
                "groups": [
                    {
                        "terms": ["messaging"],
                        "terms_depth": "core",
                        "terms_operator": "any",
                    }
                ],
            },
            year_founded=[1970, 2018],
        )
        assert_matches_type(CompanyBasic, search, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncGrata) -> None:
        response = await async_client.search.with_raw_response.create(
            authorization="Authorization",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = await response.parse()
        assert_matches_type(CompanyBasic, search, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncGrata) -> None:
        async with async_client.search.with_streaming_response.create(
            authorization="Authorization",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = await response.parse()
            assert_matches_type(CompanyBasic, search, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_similar(self, async_client: AsyncGrata) -> None:
        search = await async_client.search.similar(
            authorization="Authorization",
        )
        assert_matches_type(CompanyBasic, search, path=["response"])

    @parametrize
    async def test_method_similar_with_all_params(self, async_client: AsyncGrata) -> None:
        search = await async_client.search.similar(
            authorization="Authorization",
            business_models=["software"],
            company_uid="26AJRCR2",
            domain="slack.com",
            employees_change=[-50],
            employees_change_time="month",
            employees_on_professional_networks_range=[0],
            end_customer=["b2b", "b2c"],
            funding_size=[0, 100000000],
            funding_stage=["early_stage_funding"],
            grata_employees_estimates_range=[10, 100],
            headquarters={
                "exclude": [
                    {
                        "city": "New York",
                        "country": "United States",
                        "state": "New York",
                    }
                ],
                "include": [
                    {
                        "city": "Los Angeles ",
                        "country": "United States",
                        "state": "California",
                    }
                ],
            },
            industry_classifications={
                "exclude": [0],
                "include": [0],
            },
            is_funded=True,
            lists={
                "exclude": ["string"],
                "include": ["string"],
            },
            ownership=["bootstrapped"],
            page_token="",
            terms_exclude=["string"],
            terms_include={
                "group_operator": "any",
                "groups": [
                    {
                        "terms": ["messaging"],
                        "terms_depth": "core",
                        "terms_operator": "any",
                    }
                ],
            },
            year_founded=[1970, 2018],
        )
        assert_matches_type(CompanyBasic, search, path=["response"])

    @parametrize
    async def test_raw_response_similar(self, async_client: AsyncGrata) -> None:
        response = await async_client.search.with_raw_response.similar(
            authorization="Authorization",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = await response.parse()
        assert_matches_type(CompanyBasic, search, path=["response"])

    @parametrize
    async def test_streaming_response_similar(self, async_client: AsyncGrata) -> None:
        async with async_client.search.with_streaming_response.similar(
            authorization="Authorization",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = await response.parse()
            assert_matches_type(CompanyBasic, search, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .bulk import (
    BulkResource,
    AsyncBulkResource,
    BulkResourceWithRawResponse,
    AsyncBulkResourceWithRawResponse,
    BulkResourceWithStreamingResponse,
    AsyncBulkResourceWithStreamingResponse,
)
from .lists import (
    ListsResource,
    AsyncListsResource,
    ListsResourceWithRawResponse,
    AsyncListsResourceWithRawResponse,
    ListsResourceWithStreamingResponse,
    AsyncListsResourceWithStreamingResponse,
)
from .enrich import (
    EnrichResource,
    AsyncEnrichResource,
    EnrichResourceWithRawResponse,
    AsyncEnrichResourceWithRawResponse,
    EnrichResourceWithStreamingResponse,
    AsyncEnrichResourceWithStreamingResponse,
)
from .search import (
    SearchResource,
    AsyncSearchResource,
    SearchResourceWithRawResponse,
    AsyncSearchResourceWithRawResponse,
    SearchResourceWithStreamingResponse,
    AsyncSearchResourceWithStreamingResponse,
)

__all__ = [
    "BulkResource",
    "AsyncBulkResource",
    "BulkResourceWithRawResponse",
    "AsyncBulkResourceWithRawResponse",
    "BulkResourceWithStreamingResponse",
    "AsyncBulkResourceWithStreamingResponse",
    "EnrichResource",
    "AsyncEnrichResource",
    "EnrichResourceWithRawResponse",
    "AsyncEnrichResourceWithRawResponse",
    "EnrichResourceWithStreamingResponse",
    "AsyncEnrichResourceWithStreamingResponse",
    "ListsResource",
    "AsyncListsResource",
    "ListsResourceWithRawResponse",
    "AsyncListsResourceWithRawResponse",
    "ListsResourceWithStreamingResponse",
    "AsyncListsResourceWithStreamingResponse",
    "SearchResource",
    "AsyncSearchResource",
    "SearchResourceWithRawResponse",
    "AsyncSearchResourceWithRawResponse",
    "SearchResourceWithStreamingResponse",
    "AsyncSearchResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal

import httpx

from ..types import search_create_params, search_similar_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.company_basic import CompanyBasic

__all__ = ["SearchResource", "AsyncSearchResource"]


class SearchResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/TJC-LP/grata-python#accessing-raw-response-data-eg-headers
        """
        return SearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/TJC-LP/grata-python#with_streaming_response
        """
        return SearchResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        authorization: str,
        business_models: List[
            Literal[
                "software",
                "software_enabled",
                "services",
                "hardware",
                "content_and_publishing",
                "investment_banks_and_business_brokers",
                "education",
                "directory",
                "job_site",
                "staffing_and_recruiting",
                "private_equity_and_venture_capital",
                "private_schools",
                "retailer",
                "manufacturer",
                "distributor",
                "producer",
                "marketplace",
                "hospitals_and_medical_centers",
                "colleges_and_universities",
                "government",
                "us_federal_agencies",
                "nonprofit_and_associations",
                "religious_institutions",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        company_uid: str | NotGiven = NOT_GIVEN,
        domain: str | NotGiven = NOT_GIVEN,
        employees_change: Iterable[float] | NotGiven = NOT_GIVEN,
        employees_change_time: Literal["month", "quarter", "six_month", "annual"] | NotGiven = NOT_GIVEN,
        employees_on_professional_networks_range: Iterable[float] | NotGiven = NOT_GIVEN,
        end_customer: List[
            Literal[
                "b2b",
                "b2c",
                "information_technology",
                "professional_services",
                "electronics",
                "commercial_and_residential_services",
                "hospitality_and_leisure",
                "media",
                "finance",
                "industrials",
                "transportation",
                "education",
                "agriculture",
                "healthcare",
                "government",
                "consumer_product_and_retail",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        funding_size: Iterable[float] | NotGiven = NOT_GIVEN,
        funding_stage: List[
            Literal[
                "early_stage_funding", "late_stage_funding", "private_equity_backed", "other_funding", "pre_ipo_funding"
            ]
        ]
        | NotGiven = NOT_GIVEN,
        grata_employees_estimates_range: Iterable[float] | NotGiven = NOT_GIVEN,
        headquarters: search_create_params.Headquarters | NotGiven = NOT_GIVEN,
        industry_classifications: search_create_params.IndustryClassifications | NotGiven = NOT_GIVEN,
        is_funded: bool | NotGiven = NOT_GIVEN,
        lists: search_create_params.Lists | NotGiven = NOT_GIVEN,
        ownership: List[
            Literal[
                "bootstrapped",
                "investor_backed",
                "public",
                "public_subsidiary",
                "private_subsidiary",
                "private_equity",
                "private_equity_add_on",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        page_token: str | NotGiven = NOT_GIVEN,
        terms_exclude: List[str] | NotGiven = NOT_GIVEN,
        terms_include: search_create_params.TermsInclude | NotGiven = NOT_GIVEN,
        year_founded: Iterable[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyBasic:
        """Returns Grata-powered search results based on an input search query.

        If you're
        using any of the filters in the UI that are not presented below, the results may
        differ.

        Args:
          business_models: Business models to search on.

          company_uid: Alphanumeric Grata ID for the company (case-sensitive).

          domain: Domain of the company for similar search. Protocol and path can be included. If
              both the domain and company_uid are specified, domain will be referenced.

          employees_change: Range of % employee growth.

          employees_change_time: The interval for employee growth rate.

          employees_on_professional_networks_range: The range of employee counts listed on professional networks. Inputting 100,001
              as the maximum value will search for all employee sizes above the minimum.
              [100,100001] will search for all companies with 100 or more employees

          end_customer: End vertical that the company sells to.

          funding_size: Range of funding the company has received in USD. Ranges can only start and
              begin with the following values: 0, 5000000, 10000000, 20000000, 50000000,
              100000000, 200000000, 500000000, 500000001. 500000001 equates to maximum.

          grata_employees_estimates_range: The range of employee counts based on Grata Employee estimates. Inputting
              100,001 as the maximum value will search for all employee sizes above the
              minimum. [100,100001] will search for all companies with 100 or more employees

          headquarters: Headquarter locations supports all countries and US city/states. State cannot be
              left blank if city is populated. Country cannot be other than United States if
              searching for city/state.

          industry_classifications: Industry classification code for the company. Pass the industry NAICS code or
              Grata's specific software industry code listed in the mapping doc -
              https://grata.stoplight.io/docs/grata/branches/v1.3/42ptq2xej8i5j-software-industry-code-mapping

          is_funded: Indicates whether or not the company has received outside funding.

          lists: Grata list IDs to search within. Default logic for include is "or", default
              logic for exclude is "and."

          ownership: Ownership types to search and sort on.

          terms_exclude: Keywords to exclude from the search.

          terms_include: String used for keyword search. This is an array of keywords

          year_founded: Range of founding years.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return self._post(
            "/api/v1.4/search/",
            body=maybe_transform(
                {
                    "business_models": business_models,
                    "company_uid": company_uid,
                    "domain": domain,
                    "employees_change": employees_change,
                    "employees_change_time": employees_change_time,
                    "employees_on_professional_networks_range": employees_on_professional_networks_range,
                    "end_customer": end_customer,
                    "funding_size": funding_size,
                    "funding_stage": funding_stage,
                    "grata_employees_estimates_range": grata_employees_estimates_range,
                    "headquarters": headquarters,
                    "industry_classifications": industry_classifications,
                    "is_funded": is_funded,
                    "lists": lists,
                    "ownership": ownership,
                    "page_token": page_token,
                    "terms_exclude": terms_exclude,
                    "terms_include": terms_include,
                    "year_founded": year_founded,
                },
                search_create_params.SearchCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyBasic,
        )

    def similar(
        self,
        *,
        authorization: str,
        business_models: List[
            Literal[
                "software",
                "software_enabled",
                "services",
                "hardware",
                "content_and_publishing",
                "investment_banks_and_business_brokers",
                "education",
                "directory",
                "job_site",
                "staffing_and_recruiting",
                "private_equity_and_venture_capital",
                "private_schools",
                "retailer",
                "manufacturer",
                "distributor",
                "producer",
                "marketplace",
                "hospitals_and_medical_centers",
                "colleges_and_universities",
                "government",
                "us_federal_agencies",
                "nonprofit_and_associations",
                "religious_institutions",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        company_uid: str | NotGiven = NOT_GIVEN,
        domain: str | NotGiven = NOT_GIVEN,
        employees_change: Iterable[float] | NotGiven = NOT_GIVEN,
        employees_change_time: Literal["month", "quarter", "six_month", "annual"] | NotGiven = NOT_GIVEN,
        employees_on_professional_networks_range: Iterable[float] | NotGiven = NOT_GIVEN,
        end_customer: List[
            Literal[
                "b2b",
                "b2c",
                "information_technology",
                "professional_services",
                "electronics",
                "commercial_and_residential_services",
                "hospitality_and_leisure",
                "media",
                "finance",
                "industrials",
                "transportation",
                "education",
                "agriculture",
                "healthcare",
                "government",
                "consumer_product_and_retail",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        funding_size: Iterable[float] | NotGiven = NOT_GIVEN,
        funding_stage: List[
            Literal[
                "early_stage_funding", "late_stage_funding", "private_equity_backed", "other_funding", "pre_ipo_funding"
            ]
        ]
        | NotGiven = NOT_GIVEN,
        grata_employees_estimates_range: Iterable[float] | NotGiven = NOT_GIVEN,
        headquarters: search_similar_params.Headquarters | NotGiven = NOT_GIVEN,
        industry_classifications: search_similar_params.IndustryClassifications | NotGiven = NOT_GIVEN,
        is_funded: bool | NotGiven = NOT_GIVEN,
        lists: search_similar_params.Lists | NotGiven = NOT_GIVEN,
        ownership: List[
            Literal[
                "bootstrapped",
                "investor_backed",
                "public",
                "public_subsidiary",
                "private_subsidiary",
                "private_equity",
                "private_equity_add_on",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        page_token: str | NotGiven = NOT_GIVEN,
        terms_exclude: List[str] | NotGiven = NOT_GIVEN,
        terms_include: search_similar_params.TermsInclude | NotGiven = NOT_GIVEN,
        year_founded: Iterable[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyBasic:
        """Returns Grata-powered search results based on an input search query.

        If you're
        using any of the filters in the UI that are not presented below, the results may
        differ.

        Args:
          business_models: Business models to search on.

          company_uid: Alphanumeric Grata ID for the company (case-sensitive).

          domain: Domain of the company for similar search. Protocol and path can be included. If
              both the domain and company_uid are specified, domain will be referenced.

          employees_change: Range of % employee growth.

          employees_change_time: The interval for employee growth rate.

          employees_on_professional_networks_range: The range of employee counts listed on professional networks. Inputting 100,001
              as the maximum value will search for all employee sizes above the minimum.
              [100,100001] will search for all companies with 100 or more employees

          end_customer: End vertical that the company sells to.

          funding_size: Range of funding the company has received in USD. Ranges can only start and
              begin with the following values: 0, 5000000, 10000000, 20000000, 50000000,
              100000000, 200000000, 500000000, 500000001. 500000001 equates to maximum.

          grata_employees_estimates_range: The range of employee counts based on Grata Employee estimates. Inputting
              100,001 as the maximum value will search for all employee sizes above the
              minimum. [100,100001] will search for all companies with 100 or more employees

          headquarters: Headquarter locations supports all countries and US city/states. State cannot be
              left blank if city is populated. Country cannot be other than United States if
              searching for city/state.

          industry_classifications: Industry classification code for the company. Pass the industry NAICS code or
              Grata's specific software industry code listed in the mapping doc -
              https://grata.stoplight.io/docs/grata/branches/v1.3/42ptq2xej8i5j-software-industry-code-mapping

          is_funded: Indicates whether or not the company has received outside funding.

          lists: Grata list IDs to search within. Default logic for include is "or", default
              logic for exclude is "and."

          ownership: Ownership types to search and sort on.

          terms_exclude: Keywords to exclude from the search.

          terms_include: String used for keyword search. This is an array of keywords

          year_founded: Range of founding years.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return self._post(
            "/api/v1.4/search-similar/",
            body=maybe_transform(
                {
                    "business_models": business_models,
                    "company_uid": company_uid,
                    "domain": domain,
                    "employees_change": employees_change,
                    "employees_change_time": employees_change_time,
                    "employees_on_professional_networks_range": employees_on_professional_networks_range,
                    "end_customer": end_customer,
                    "funding_size": funding_size,
                    "funding_stage": funding_stage,
                    "grata_employees_estimates_range": grata_employees_estimates_range,
                    "headquarters": headquarters,
                    "industry_classifications": industry_classifications,
                    "is_funded": is_funded,
                    "lists": lists,
                    "ownership": ownership,
                    "page_token": page_token,
                    "terms_exclude": terms_exclude,
                    "terms_include": terms_include,
                    "year_founded": year_founded,
                },
                search_similar_params.SearchSimilarParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyBasic,
        )


class AsyncSearchResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/TJC-LP/grata-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/TJC-LP/grata-python#with_streaming_response
        """
        return AsyncSearchResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        authorization: str,
        business_models: List[
            Literal[
                "software",
                "software_enabled",
                "services",
                "hardware",
                "content_and_publishing",
                "investment_banks_and_business_brokers",
                "education",
                "directory",
                "job_site",
                "staffing_and_recruiting",
                "private_equity_and_venture_capital",
                "private_schools",
                "retailer",
                "manufacturer",
                "distributor",
                "producer",
                "marketplace",
                "hospitals_and_medical_centers",
                "colleges_and_universities",
                "government",
                "us_federal_agencies",
                "nonprofit_and_associations",
                "religious_institutions",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        company_uid: str | NotGiven = NOT_GIVEN,
        domain: str | NotGiven = NOT_GIVEN,
        employees_change: Iterable[float] | NotGiven = NOT_GIVEN,
        employees_change_time: Literal["month", "quarter", "six_month", "annual"] | NotGiven = NOT_GIVEN,
        employees_on_professional_networks_range: Iterable[float] | NotGiven = NOT_GIVEN,
        end_customer: List[
            Literal[
                "b2b",
                "b2c",
                "information_technology",
                "professional_services",
                "electronics",
                "commercial_and_residential_services",
                "hospitality_and_leisure",
                "media",
                "finance",
                "industrials",
                "transportation",
                "education",
                "agriculture",
                "healthcare",
                "government",
                "consumer_product_and_retail",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        funding_size: Iterable[float] | NotGiven = NOT_GIVEN,
        funding_stage: List[
            Literal[
                "early_stage_funding", "late_stage_funding", "private_equity_backed", "other_funding", "pre_ipo_funding"
            ]
        ]
        | NotGiven = NOT_GIVEN,
        grata_employees_estimates_range: Iterable[float] | NotGiven = NOT_GIVEN,
        headquarters: search_create_params.Headquarters | NotGiven = NOT_GIVEN,
        industry_classifications: search_create_params.IndustryClassifications | NotGiven = NOT_GIVEN,
        is_funded: bool | NotGiven = NOT_GIVEN,
        lists: search_create_params.Lists | NotGiven = NOT_GIVEN,
        ownership: List[
            Literal[
                "bootstrapped",
                "investor_backed",
                "public",
                "public_subsidiary",
                "private_subsidiary",
                "private_equity",
                "private_equity_add_on",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        page_token: str | NotGiven = NOT_GIVEN,
        terms_exclude: List[str] | NotGiven = NOT_GIVEN,
        terms_include: search_create_params.TermsInclude | NotGiven = NOT_GIVEN,
        year_founded: Iterable[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyBasic:
        """Returns Grata-powered search results based on an input search query.

        If you're
        using any of the filters in the UI that are not presented below, the results may
        differ.

        Args:
          business_models: Business models to search on.

          company_uid: Alphanumeric Grata ID for the company (case-sensitive).

          domain: Domain of the company for similar search. Protocol and path can be included. If
              both the domain and company_uid are specified, domain will be referenced.

          employees_change: Range of % employee growth.

          employees_change_time: The interval for employee growth rate.

          employees_on_professional_networks_range: The range of employee counts listed on professional networks. Inputting 100,001
              as the maximum value will search for all employee sizes above the minimum.
              [100,100001] will search for all companies with 100 or more employees

          end_customer: End vertical that the company sells to.

          funding_size: Range of funding the company has received in USD. Ranges can only start and
              begin with the following values: 0, 5000000, 10000000, 20000000, 50000000,
              100000000, 200000000, 500000000, 500000001. 500000001 equates to maximum.

          grata_employees_estimates_range: The range of employee counts based on Grata Employee estimates. Inputting
              100,001 as the maximum value will search for all employee sizes above the
              minimum. [100,100001] will search for all companies with 100 or more employees

          headquarters: Headquarter locations supports all countries and US city/states. State cannot be
              left blank if city is populated. Country cannot be other than United States if
              searching for city/state.

          industry_classifications: Industry classification code for the company. Pass the industry NAICS code or
              Grata's specific software industry code listed in the mapping doc -
              https://grata.stoplight.io/docs/grata/branches/v1.3/42ptq2xej8i5j-software-industry-code-mapping

          is_funded: Indicates whether or not the company has received outside funding.

          lists: Grata list IDs to search within. Default logic for include is "or", default
              logic for exclude is "and."

          ownership: Ownership types to search and sort on.

          terms_exclude: Keywords to exclude from the search.

          terms_include: String used for keyword search. This is an array of keywords

          year_founded: Range of founding years.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return await self._post(
            "/api/v1.4/search/",
            body=await async_maybe_transform(
                {
                    "business_models": business_models,
                    "company_uid": company_uid,
                    "domain": domain,
                    "employees_change": employees_change,
                    "employees_change_time": employees_change_time,
                    "employees_on_professional_networks_range": employees_on_professional_networks_range,
                    "end_customer": end_customer,
                    "funding_size": funding_size,
                    "funding_stage": funding_stage,
                    "grata_employees_estimates_range": grata_employees_estimates_range,
                    "headquarters": headquarters,
                    "industry_classifications": industry_classifications,
                    "is_funded": is_funded,
                    "lists": lists,
                    "ownership": ownership,
                    "page_token": page_token,
                    "terms_exclude": terms_exclude,
                    "terms_include": terms_include,
                    "year_founded": year_founded,
                },
                search_create_params.SearchCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyBasic,
        )

    async def similar(
        self,
        *,
        authorization: str,
        business_models: List[
            Literal[
                "software",
                "software_enabled",
                "services",
                "hardware",
                "content_and_publishing",
                "investment_banks_and_business_brokers",
                "education",
                "directory",
                "job_site",
                "staffing_and_recruiting",
                "private_equity_and_venture_capital",
                "private_schools",
                "retailer",
                "manufacturer",
                "distributor",
                "producer",
                "marketplace",
                "hospitals_and_medical_centers",
                "colleges_and_universities",
                "government",
                "us_federal_agencies",
                "nonprofit_and_associations",
                "religious_institutions",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        company_uid: str | NotGiven = NOT_GIVEN,
        domain: str | NotGiven = NOT_GIVEN,
        employees_change: Iterable[float] | NotGiven = NOT_GIVEN,
        employees_change_time: Literal["month", "quarter", "six_month", "annual"] | NotGiven = NOT_GIVEN,
        employees_on_professional_networks_range: Iterable[float] | NotGiven = NOT_GIVEN,
        end_customer: List[
            Literal[
                "b2b",
                "b2c",
                "information_technology",
                "professional_services",
                "electronics",
                "commercial_and_residential_services",
                "hospitality_and_leisure",
                "media",
                "finance",
                "industrials",
                "transportation",
                "education",
                "agriculture",
                "healthcare",
                "government",
                "consumer_product_and_retail",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        funding_size: Iterable[float] | NotGiven = NOT_GIVEN,
        funding_stage: List[
            Literal[
                "early_stage_funding", "late_stage_funding", "private_equity_backed", "other_funding", "pre_ipo_funding"
            ]
        ]
        | NotGiven = NOT_GIVEN,
        grata_employees_estimates_range: Iterable[float] | NotGiven = NOT_GIVEN,
        headquarters: search_similar_params.Headquarters | NotGiven = NOT_GIVEN,
        industry_classifications: search_similar_params.IndustryClassifications | NotGiven = NOT_GIVEN,
        is_funded: bool | NotGiven = NOT_GIVEN,
        lists: search_similar_params.Lists | NotGiven = NOT_GIVEN,
        ownership: List[
            Literal[
                "bootstrapped",
                "investor_backed",
                "public",
                "public_subsidiary",
                "private_subsidiary",
                "private_equity",
                "private_equity_add_on",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        page_token: str | NotGiven = NOT_GIVEN,
        terms_exclude: List[str] | NotGiven = NOT_GIVEN,
        terms_include: search_similar_params.TermsInclude | NotGiven = NOT_GIVEN,
        year_founded: Iterable[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CompanyBasic:
        """Returns Grata-powered search results based on an input search query.

        If you're
        using any of the filters in the UI that are not presented below, the results may
        differ.

        Args:
          business_models: Business models to search on.

          company_uid: Alphanumeric Grata ID for the company (case-sensitive).

          domain: Domain of the company for similar search. Protocol and path can be included. If
              both the domain and company_uid are specified, domain will be referenced.

          employees_change: Range of % employee growth.

          employees_change_time: The interval for employee growth rate.

          employees_on_professional_networks_range: The range of employee counts listed on professional networks. Inputting 100,001
              as the maximum value will search for all employee sizes above the minimum.
              [100,100001] will search for all companies with 100 or more employees

          end_customer: End vertical that the company sells to.

          funding_size: Range of funding the company has received in USD. Ranges can only start and
              begin with the following values: 0, 5000000, 10000000, 20000000, 50000000,
              100000000, 200000000, 500000000, 500000001. 500000001 equates to maximum.

          grata_employees_estimates_range: The range of employee counts based on Grata Employee estimates. Inputting
              100,001 as the maximum value will search for all employee sizes above the
              minimum. [100,100001] will search for all companies with 100 or more employees

          headquarters: Headquarter locations supports all countries and US city/states. State cannot be
              left blank if city is populated. Country cannot be other than United States if
              searching for city/state.

          industry_classifications: Industry classification code for the company. Pass the industry NAICS code or
              Grata's specific software industry code listed in the mapping doc -
              https://grata.stoplight.io/docs/grata/branches/v1.3/42ptq2xej8i5j-software-industry-code-mapping

          is_funded: Indicates whether or not the company has received outside funding.

          lists: Grata list IDs to search within. Default logic for include is "or", default
              logic for exclude is "and."

          ownership: Ownership types to search and sort on.

          terms_exclude: Keywords to exclude from the search.

          terms_include: String used for keyword search. This is an array of keywords

          year_founded: Range of founding years.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Authorization": authorization, **(extra_headers or {})}
        return await self._post(
            "/api/v1.4/search-similar/",
            body=await async_maybe_transform(
                {
                    "business_models": business_models,
                    "company_uid": company_uid,
                    "domain": domain,
                    "employees_change": employees_change,
                    "employees_change_time": employees_change_time,
                    "employees_on_professional_networks_range": employees_on_professional_networks_range,
                    "end_customer": end_customer,
                    "funding_size": funding_size,
                    "funding_stage": funding_stage,
                    "grata_employees_estimates_range": grata_employees_estimates_range,
                    "headquarters": headquarters,
                    "industry_classifications": industry_classifications,
                    "is_funded": is_funded,
                    "lists": lists,
                    "ownership": ownership,
                    "page_token": page_token,
                    "terms_exclude": terms_exclude,
                    "terms_include": terms_include,
                    "year_founded": year_founded,
                },
                search_similar_params.SearchSimilarParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CompanyBasic,
        )


class SearchResourceWithRawResponse:
    def __init__(self, search: SearchResource) -> None:
        self._search = search

        self.create = to_raw_response_wrapper(
            search.create,
        )
        self.similar = to_raw_response_wrapper(
            search.similar,
        )


class AsyncSearchResourceWithRawResponse:
    def __init__(self, search: AsyncSearchResource) -> None:
        self._search = search

        self.create = async_to_raw_response_wrapper(
            search.create,
        )
        self.similar = async_to_raw_response_wrapper(
            search.similar,
        )


class SearchResourceWithStreamingResponse:
    def __init__(self, search: SearchResource) -> None:
        self._search = search

        self.create = to_streamed_response_wrapper(
            search.create,
        )
        self.similar = to_streamed_response_wrapper(
            search.similar,
        )


class AsyncSearchResourceWithStreamingResponse:
    def __init__(self, search: AsyncSearchResource) -> None:
        self._search = search

        self.create = async_to_streamed_response_wrapper(
            search.create,
        )
        self.similar = async_to_streamed_response_wrapper(
            search.similar,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from grata import Grata, AsyncGrata
from tests.utils import assert_matches_type
from grata.types.shared import CompanyDetailed

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestLists:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Grata) -> None:
        list_ = client.lists.create(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Grata) -> None:
        list_ = client.lists.create(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            name="New List",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Grata) -> None:
        response = client.lists.with_raw_response.create(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        list_ = response.parse()
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Grata) -> None:
        with client.lists.with_streaming_response.create(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            list_ = response.parse()
            assert_matches_type(CompanyDetailed, list_, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Grata) -> None:
        list_ = client.lists.retrieve(
            list_uid="ZTZGNNT7",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Grata) -> None:
        response = client.lists.with_raw_response.retrieve(
            list_uid="ZTZGNNT7",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        list_ = response.parse()
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Grata) -> None:
        with client.lists.with_streaming_response.retrieve(
            list_uid="ZTZGNNT7",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            list_ = response.parse()
            assert_matches_type(CompanyDetailed, list_, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Grata) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `list_uid` but received ''"):
            client.lists.with_raw_response.retrieve(
                list_uid="",
                authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            )

    @parametrize
    def test_method_update(self, client: Grata) -> None:
        list_ = client.lists.update(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Grata) -> None:
        list_ = client.lists.update(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            name="New List Name",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Grata) -> None:
        response = client.lists.with_raw_response.update(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        list_ = response.parse()
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Grata) -> None:
        with client.lists.with_streaming_response.update(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            list_ = response.parse()
            assert_matches_type(CompanyDetailed, list_, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Grata) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `list_uid` but received ''"):
            client.lists.with_raw_response.update(
                list_uid="",
                authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            )

    @parametrize
    def test_method_list(self, client: Grata) -> None:
        list_ = client.lists.list(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Grata) -> None:
        list_ = client.lists.list(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            name="name=List%20A",
            page="page=2",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Grata) -> None:
        response = client.lists.with_raw_response.list(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        list_ = response.parse()
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Grata) -> None:
        with client.lists.with_streaming_response.list(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            list_ = response.parse()
            assert_matches_type(CompanyDetailed, list_, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Grata) -> None:
        list_ = client.lists.delete(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert list_ is None

    @parametrize
    def test_raw_response_delete(self, client: Grata) -> None:
        response = client.lists.with_raw_response.delete(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        list_ = response.parse()
        assert list_ is None

    @parametrize
    def test_streaming_response_delete(self, client: Grata) -> None:
        with client.lists.with_streaming_response.delete(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            list_ = response.parse()
            assert list_ is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Grata) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `list_uid` but received ''"):
            client.lists.with_raw_response.delete(
                list_uid="",
                authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            )

    @parametrize
    def test_method_companies(self, client: Grata) -> None:
        list_ = client.lists.companies(
            list_uid="XYZ12345",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_method_companies_with_all_params(self, client: Grata) -> None:
        list_ = client.lists.companies(
            list_uid="XYZ12345",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            action="add",
            domains=["grata.com"],
            uids=["XYZ12356"],
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_raw_response_companies(self, client: Grata) -> None:
        response = client.lists.with_raw_response.companies(
            list_uid="XYZ12345",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        list_ = response.parse()
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    def test_streaming_response_companies(self, client: Grata) -> None:
        with client.lists.with_streaming_response.companies(
            list_uid="XYZ12345",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            list_ = response.parse()
            assert_matches_type(CompanyDetailed, list_, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_companies(self, client: Grata) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `list_uid` but received ''"):
            client.lists.with_raw_response.companies(
                list_uid="",
                authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            )


class TestAsyncLists:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncGrata) -> None:
        list_ = await async_client.lists.create(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncGrata) -> None:
        list_ = await async_client.lists.create(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            name="New List",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncGrata) -> None:
        response = await async_client.lists.with_raw_response.create(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        list_ = await response.parse()
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncGrata) -> None:
        async with async_client.lists.with_streaming_response.create(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            list_ = await response.parse()
            assert_matches_type(CompanyDetailed, list_, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncGrata) -> None:
        list_ = await async_client.lists.retrieve(
            list_uid="ZTZGNNT7",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncGrata) -> None:
        response = await async_client.lists.with_raw_response.retrieve(
            list_uid="ZTZGNNT7",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        list_ = await response.parse()
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncGrata) -> None:
        async with async_client.lists.with_streaming_response.retrieve(
            list_uid="ZTZGNNT7",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            list_ = await response.parse()
            assert_matches_type(CompanyDetailed, list_, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncGrata) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `list_uid` but received ''"):
            await async_client.lists.with_raw_response.retrieve(
                list_uid="",
                authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncGrata) -> None:
        list_ = await async_client.lists.update(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncGrata) -> None:
        list_ = await async_client.lists.update(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            name="New List Name",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncGrata) -> None:
        response = await async_client.lists.with_raw_response.update(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        list_ = await response.parse()
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncGrata) -> None:
        async with async_client.lists.with_streaming_response.update(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            list_ = await response.parse()
            assert_matches_type(CompanyDetailed, list_, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncGrata) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `list_uid` but received ''"):
            await async_client.lists.with_raw_response.update(
                list_uid="",
                authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncGrata) -> None:
        list_ = await async_client.lists.list(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncGrata) -> None:
        list_ = await async_client.lists.list(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            name="name=List%20A",
            page="page=2",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncGrata) -> None:
        response = await async_client.lists.with_raw_response.list(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        list_ = await response.parse()
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncGrata) -> None:
        async with async_client.lists.with_streaming_response.list(
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            list_ = await response.parse()
            assert_matches_type(CompanyDetailed, list_, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncGrata) -> None:
        list_ = await async_client.lists.delete(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert list_ is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncGrata) -> None:
        response = await async_client.lists.with_raw_response.delete(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        list_ = await response.parse()
        assert list_ is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncGrata) -> None:
        async with async_client.lists.with_streaming_response.delete(
            list_uid="XY2Z8V23",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            list_ = await response.parse()
            assert list_ is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncGrata) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `list_uid` but received ''"):
            await async_client.lists.with_raw_response.delete(
                list_uid="",
                authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            )

    @parametrize
    async def test_method_companies(self, async_client: AsyncGrata) -> None:
        list_ = await async_client.lists.companies(
            list_uid="XYZ12345",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_method_companies_with_all_params(self, async_client: AsyncGrata) -> None:
        list_ = await async_client.lists.companies(
            list_uid="XYZ12345",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            action="add",
            domains=["grata.com"],
            uids=["XYZ12356"],
        )
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_raw_response_companies(self, async_client: AsyncGrata) -> None:
        response = await async_client.lists.with_raw_response.companies(
            list_uid="XYZ12345",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        list_ = await response.parse()
        assert_matches_type(CompanyDetailed, list_, path=["response"])

    @parametrize
    async def test_streaming_response_companies(self, async_client: AsyncGrata) -> None:
        async with async_client.lists.with_streaming_response.companies(
            list_uid="XYZ12345",
            authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            list_ = await response.parse()
            assert_matches_type(CompanyDetailed, list_, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_companies(self, async_client: AsyncGrata) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `list_uid` but received ''"):
            await async_client.lists.with_raw_response.companies(
                list_uid="",
                authorization="Token 840cda398b02093940807af4885853500c1cf5bb",
            )
